﻿//*****************************************************************************
// Description     : Declaration and macro definition for SiTCP communication
// Target Device   : APU101
// Target Platform : Windows, .NET Framework, C++
// Date            : July 2017
// Version         : 1.0
//*****************************************************************************
#ifndef __SITCP_H__
#define __SITCP_H__

#include "sys_compatible.h"
using namespace System;
using namespace System::Net;
using namespace System::Net::Sockets;


//******************** macro definition ********************
// command communication: header common part
#define SITCP_VER_TYPE              ((unsigned char)(0xff))  // version, type

// command communication: read command flag
#define SITCP_READ_REQ_CMDFLAG      ((unsigned char)(0xc0))  // request
#define SITCP_READ_ACK_CMDFLAG      ((unsigned char)(0xc8))  // acknowledge

// command communication: write command flag
#define SITCP_WRITE_REQ_CMDFLAG     ((unsigned char)(0x80))  // request
#define SITCP_WRITE_ACK_CMDFLAG     ((unsigned char)(0x88))  // acknowledge

#define SITCP_MAX_CMD_DATABYTE      (255)    // maximum size of data


//******************** structure type definition ********************
// header for command communication
typedef struct sitcp_header {
    unsigned char ver_type;      // version(4bit), type(4bit)
    unsigned char cmd_flag;      // command(4bit), flag(4bit)
    unsigned char id;            // identifier(0-255)
    unsigned char length;        // data length(byte)
    unsigned int  address;       // command address(32bit)
} SITCP_HEADER;

// data format for command communication
typedef struct sitcp_format {
    SITCP_HEADER        header;
    union {
        unsigned char   u08[SITCP_MAX_CMD_DATABYTE/1];    // unit of 8bit
        unsigned short  u16[SITCP_MAX_CMD_DATABYTE/2];    // unit of 16bit
        unsigned int    u32[SITCP_MAX_CMD_DATABYTE/4];    // unit of 32bit
        unsigned qword  u64[SITCP_MAX_CMD_DATABYTE/8];    // unit of 64bit
    } data;
} SITCP_FORMAT;


//******************** class definition ********************
// encapsulate class for reference all instances for communication
public ref class SiTCPcom {
  public:
    UdpClient ^cmdClient;   // for command communication
    TcpClient ^dataClient;  // for fast data transfer
};


//******************** prototype declaration ********************
extern int sitcp_connect(SiTCPcom^);
extern void sitcp_disconnect(SiTCPcom^);

extern unsigned short sitcp_read_16bit(SiTCPcom^, unsigned int, int*);
extern unsigned int   sitcp_read_32bit(SiTCPcom^, unsigned int, int*);
extern unsigned qword sitcp_read_48bit(SiTCPcom^, unsigned int, int*);
extern unsigned qword sitcp_read_64bit(SiTCPcom^, unsigned int, int*);

extern int sitcp_write_16bit(SiTCPcom^, unsigned int, unsigned short);
extern int sitcp_write_32bit(SiTCPcom^, unsigned int, unsigned int);
extern int sitcp_write_48bit(SiTCPcom^, unsigned int, unsigned qword);
extern int sitcp_write_64bit(SiTCPcom^, unsigned int, unsigned qword);

extern int sitcp_recv_data(SiTCPcom^, char*, int, int);


#endif // __SITCP_H__

